@@ -18,7 +18,7 @@ License : GNU General Public License Version 2 (See COPYING)
 """
 
 __docformat__ = 'restructuredtext'
-__revision__ = '$Revision: 166 $'[11:-1].strip()
+__revision__ = '$Revision: 185 $'[11:-1].strip()
 
 
 import inspect
@@ -97,6 +97,7 @@ class Yolk(object):
         self.version = ""
         #List of all versions not hidden on PyPI
         self.all_versions = []
+        self.pkg_spec = []
         self.options = None
         self.logger = logging.getLogger("yolk")
 
@@ -159,74 +160,68 @@ class Yolk(object):
         pkg_spec = validate_pypi_opts(opt_parser)
         if not pkg_spec:
             pkg_spec = remaining_args
-        if not self.options.search and (len(sys.argv) == 1 or\
+        self.pkg_spec = pkg_spec
+
+        if not self.options.pypi_search and (len(sys.argv) == 1 or\
                 len(remaining_args) > 2):
             opt_parser.print_help()
             return 2
 
-        if self.options.entry_points:
-            return self.show_entry_points()
-        if self.options.entry_map:
-            return self.show_entry_map()
-
         #Options that depend on querying installed packages, not PyPI.
         #We find the proper case for package names if they are installed,
         #otherwise PyPI returns the correct case.
-        if self.options.depends or self.options.all or \
-                self.options.active or self.options.nonactive  or \
+        if self.options.show_deps or self.options.show_all or \
+                self.options.show_active or self.options.show_non_active  or \
                 (self.options.show_updates and pkg_spec):
             want_installed = True
         else:
             want_installed = False
+        #show_updates may or may not have a pkg_spec 
         if not want_installed or self.options.show_updates:
             self.pypi = CheeseShop(self.options.debug)
             #XXX: We should return 2 here if we couldn't create xmlrpc server
+
         if pkg_spec:
             (self.project_name, self.version, self.all_versions) = \
-                    self.parse_pkg_ver(pkg_spec, want_installed)
+                    self.parse_pkg_ver(want_installed)
             if want_installed and not self.project_name:
                 logger.error("%s is not installed." % pkg_spec[0])
-                return 2
+                return 1
 
-        if self.options.search:
-            status = self.pypi_search(pkg_spec)
-        elif self.options.changelog:
-            status = self.show_pypi_changelog(self.options.changelog)
-        elif self.options.releases:
-            status = self.show_pypi_releases(self.options.releases)
-        elif self.options.fetch:
-            directory = "."
-            status = self.fetch(directory)
-        elif self.options.version:
-            print "yolk version %s (rev. %s)" % (VERSION, __revision__)
-            status = 0
-        elif self.options.depends:
-            status = self.show_deps()
-        elif self.options.all:
-            if self.options.active or self.options.nonactive:
-                opt_parser.error("Choose either -l, -n or -a")
-            status = self.show_distributions("all")
-        elif self.options.active:
-            if self.options.all or self.options.nonactive:
-                opt_parser.error("Choose either -l, -n or -a")
-            status = self.show_distributions("active")
-        elif self.options.nonactive:
-            if self.options.active or self.options.all:
-                opt_parser.error("Choose either -l, -n or -a")
-            status = self.show_distributions("nonactive")
-        elif self.options.versions_available:
-            status = self.get_all_versions_pypi()
-        elif self.options.browse_website:
-            status = self.browse_website()
-        elif self.options.download_links:
-            status = self.show_download_links()
-        elif self.options.show_updates:
-            status = self.show_updates()
-        elif self.options.query_metadata_pypi:
-            status = self.show_pkg_metadata_pypi()
-        else:
-            opt_parser.print_help()
-            status = 2
+        #I could prefix all these with 'cmd_' and the methods also
+        #and then iterate over the `options` dictionary keys...
+        commands = ['show_deps', 'query_metadata_pypi', 'fetch',
+                'versions_available', 'show_updates', 'browse_website',
+                'show_download_links', 'pypi_search', 'show_pypi_changelog',
+                'show_pypi_releases', 'yolk_version', 'show_all',
+                'show_active', 'show_non_active', 'show_entry_map',
+                'show_entry_points']
+
+        #Run first command it finds, and only the first command, then return
+        #XXX: Check if more than one command was set in options and give error?
+        for action in commands:
+            if getattr(self.options, action):
+                return getattr(self, action)()
+        opt_parser.print_help()
+
+
+    def show_active(self):
+        """
+        Show installed active packages
+        """
+        return self.show_distributions("active")
+
+    def show_non_active(self):
+        """
+        Show installed non-active packages
+        """
+        return self.show_distributions("nonactive")
+
+    def show_all(self):
+        """
+        Show all installed packages
+        """
+        return self.show_distributions("all")
 
     def show_updates(self):
         """
@@ -267,8 +262,10 @@ class Yolk(object):
                             found = True
                             print " %s %s (%s)" % (project_name, dist.version,
                                     newest)
-        if not found:
-            self.logger.info("You have the latest version.")
+        if not found and self.project_name:
+            self.logger.info("You have the latest version installed.")
+        elif not found:
+            self.logger.info("No newer packages found at The Cheese Shop")
         return 0
 
 
@@ -317,7 +314,7 @@ class Yolk(object):
                     add_column_text += my_plugin.add_column(dist) + " "
                 self.print_metadata(metadata, develop, active, add_column_text)
             else:
-                print dist + " has no metadata"
+                print str(dist) + " has no metadata"
             results = True
         if not results and self.project_name:
             if self.version:
@@ -357,12 +354,11 @@ class Yolk(object):
         
         """
         show_metadata = self.options.metadata
-        fields = self.options.fields
-
+        fields = self.options.fields.split(',')
+        fields = map(str.strip, fields)
         version = metadata['Version']
 
         #When showing all packages, note which are not active:
-
         if active:
             if fields:
                 active_status = ""
@@ -385,23 +381,18 @@ class Yolk(object):
             print '%s (%s)%s %s' % (metadata['Name'], version, active_status,
                                     development_status)
         else:
-
             # Need intelligent justification
-
             print metadata['Name'].ljust(15) + " - " + version.ljust(12) + \
                 " - " + status
         if fields:
-
-            #Only show specific fields
-
+            #Only show specific fields, using case-insensitive search
+            fields = map(str.lower, fields)
             for field in metadata.keys():
-                if field in fields:
+                if field.lower() in fields:
                     print '    %s: %s' % (field, metadata[field])
             print
         elif show_metadata:
-
             #Print all available metadata fields
-
             for field in metadata.keys():
                 if field != 'Name' and field != 'Summary':
                     print '    %s: %s' % (field, metadata[field])
@@ -410,10 +401,7 @@ class Yolk(object):
         """
         Show dependencies for package(s)
 
-        @param pkg_ver: setuptools pkgspec (e.g. kid>=0.8)
-        @type pkg_ver: string
-        
-        @returns: None or 2 if error 
+        @returns: 0 - sucess  1 - No dependency info supplied
         """
 
         pkgs = pkg_resources.Environment()
@@ -434,18 +422,17 @@ class Yolk(object):
             else:
                 self.logger.info(\
                     "No dependency information was supplied with the package.")
+                return 1
         return 0
 
-    def show_pypi_changelog(self, hours):
+    def show_pypi_changelog(self):
         """
         Show detailed PyPI ChangeLog for the last `hours`
 
-        @param hours: Number of `hours` to check back
-        @type hours: string
-
-        @returns: 1 if failed to retrieve from XML-RPC server
+        @returns: 0 = sucess or 1 if failed to retrieve from XML-RPC server
 
         """
+        hours = self.options.show_pypi_changelog
         try:
             changelog = self.pypi.changelog(int(hours))
         except XMLRPCFault, err_msg:
@@ -464,18 +451,16 @@ class Yolk(object):
          
         return 0
 
-    def show_pypi_releases(self, hours):
+    def show_pypi_releases(self):
         """
-        Show PyPI releases for last the `hours`
+        Show PyPI releases for the last number of `hours`
 
-        @param hours: Number of `hours` to check back
-        @type hours: string
-
-        @returns: 1 if failed to retrieve from XML-RPC server
+        @returns: 0 = success or 1 if failed to retrieve from XML-RPC server
 
         """
+        hours = int(self.options.show_pypi_releases)
         try:
-            latest_releases = self.pypi.updated_releases(int(hours))
+            latest_releases = self.pypi.updated_releases(hours)
         except XMLRPCFault, err_msg:
             self.logger.error(err_msg)
             self.logger.error("ERROR: Couldn't retrieve latest releases.")
@@ -489,7 +474,7 @@ class Yolk(object):
         """
         Query PyPI for pkg download URI for a packge
 
-        @returns: None
+        @returns: 0
         
         """
         #In case they specify version as 'dev' instead of using -T svn,
@@ -508,7 +493,7 @@ class Yolk(object):
             #Search for source, egg, and svn
             self.print_download_uri(version, True)
             self.print_download_uri(version, False)
-            self.print_download_uri("dev", True, True)
+            self.print_download_uri("dev", True)
         else:
             if self.options.file_type == "source":
                 source = True
@@ -517,15 +502,19 @@ class Yolk(object):
             self.print_download_uri(version, source)
         return 0
 
-    def print_download_uri(self, version, source, svn=False):
+    def print_download_uri(self, version, source):
         """
+        @param version: version number or 'dev' for svn
+        @type version: string
+
         @param source: download source or egg
         @type source: boolean
 
         @returns: None
 
         """
-        if svn or version == "dev":
+
+        if version == "dev":
             pkg_type = "subversion"
             source = True
         elif source:
@@ -534,21 +523,23 @@ class Yolk(object):
             pkg_type = "egg"
 
         #Use setuptools monkey-patch to grab url
-        url = get_download_uri(self.project_name, version, source)
+        url = get_download_uri(self.project_name, version, source,
+                self.options.pypi_index)
         if url:
             print "%s" % url
         else:
             self.logger.info("No download URL found for %s" % pkg_type)
 
-    def fetch(self, directory):
+    def fetch(self):
         """
         Download a package
 
-        @returns: None
+        @returns: 0 = success or 1 if failed download
         
         """
         #Default type to download
         source = True
+        directory = "."
 
         if self.options.file_type == "svn":
             version = "dev"
@@ -556,13 +547,12 @@ class Yolk(object):
                     "dev", True)
             if svn_uri:
                 directory = self.project_name + "_svn"
-                self.fetch_svn(svn_uri, directory)
-                return
+                return self.fetch_svn(svn_uri, directory)
             else:
                 self.logger.error(\
                     "ERROR: No subversion repository found for %s" % \
                     self.project_name)
-                sys.exit(2)
+                return 1
         elif self.options.file_type == "source":
             source = True
         elif self.options.file_type == "egg":
@@ -570,57 +560,74 @@ class Yolk(object):
 
         uri = get_download_uri(self.project_name, self.version, source)
         if uri:
-            self.fetch_uri(directory, uri)
+            return self.fetch_uri(directory, uri)
         else:
             self.logger.error("No %s URI found for package: %s " % \
                     (self.options.file_type, self.project_name))
-            return 2
-
+            return 1
 
     def fetch_uri(self, directory, uri):
         """
         Use ``urllib.urlretrieve`` to download package to file in sandbox dir.
+
+        @param directory: directory to download to
+        @type directory: string
+
+        @param uri: uri to download
+        @type uri: string
+
+        @returns: 0 = success or 1 for failed download
         """
-        filename = os.path.basename(urlparse(uri).path)
+        filename = os.path.basename(urlparse(uri)[2])
         if os.path.exists(filename):
             self.logger.error("ERROR: File exists: " + filename)
-            sys.exit(2)
+            return 1
 
         try:
             downloaded_filename, headers = urlretrieve(uri, filename)
-            print "Downloaded ./" + filename
+            self.logger.info("Downloaded ./" + filename)
         except IOError, err_msg:
             self.logger.error("Error downloading package %s from URL %s"  \
                     % (filename, uri))
             self.logger.error(str(err_msg))
-            sys.exit(2)
+            return 1
 
         if headers.gettype() in ["text/html"]:
             dfile = open(downloaded_filename)
             if re.search("404 Not Found", "".join(dfile.readlines())):
                 dfile.close()
                 self.logger.error("'404 Not Found' error")
-                sys.exit(2)
+                return 1
             dfile.close()
+        return 0
 
 
     def fetch_svn(self, svn_uri, directory):
         """
         Fetch subversion repository
+        
+        @param svn_uri: subversion repository uri to check out
+        @type svn_uri: string
+
+        @param directory: directory to download to
+        @type directory: string
+
+        @returns: 0 = success or 1 for failed download
+        
 
         """
         if not command_successful("svn --version"):
             self.logger.error("ERROR: Do you have subversion installed?")
-            sys.exit(2)
+            return 1
         if os.path.exists(directory):
             self.logger.error("ERROR: Checkout directory exists - %s" \
                     % directory)
-            sys.exit(2)
+            return 1
         try:
             os.mkdir(directory)
         except OSError, err_msg:
             self.logger.error("ERROR: " + str(err_msg))
-            sys.exit(2)
+            return 1
         cwd = os.path.realpath(os.curdir)
         os.chdir(directory)
         self.logger.info("Doing subversion checkout for %s" % svn_uri)
@@ -629,12 +636,16 @@ class Yolk(object):
         os.chdir(cwd)
         self.logger.info("subversion checkout is in directory './%s'" \
                 % directory)
+        return 0
 
     def browse_website(self, browser=None):
         """
         Launch web browser at project's homepage
 
-        @returns None
+        @param browser: name of web browser to use
+        @type browser: string
+
+        @returns: 0 if homepage found, 1 if no homepage found
         """
         if len(self.all_versions):
             metadata = self.pypi.release_data(self.project_name, \
@@ -648,16 +659,17 @@ class Yolk(object):
                 else:
                     browser = webbrowser.get()
                     browser.open(metadata["home_page"], 2)
-                return
+                return 0
 
         self.logger.error("No homepage URL found.")
+        return 1
 
 
-    def show_pkg_metadata_pypi(self):
+    def query_metadata_pypi(self):
         """
         Show pkg metadata queried from PyPI
 
-        @returns: None or 2 if error 
+        @returns: 0
         
         """
         if self.version and self.version in self.all_versions:
@@ -674,7 +686,7 @@ class Yolk(object):
                     print "%s: %s" % (key, metadata[key])
         return 0
 
-    def get_all_versions_pypi(self):
+    def versions_available(self):
         """
         Query PyPI for a particular version or all versions of a package
 
@@ -688,10 +700,8 @@ class Yolk(object):
 
         if self.all_versions and self.version in self.all_versions:
             print_pkg_versions(self.project_name, [self.version])
-            return 0
         elif not self.version and self.all_versions:
             print_pkg_versions(self.project_name, self.all_versions)
-            return 0
         else:
             if self.version:
                 self.logger.error("No pacakge found for version %s" \
@@ -699,6 +709,7 @@ class Yolk(object):
             else:
                 self.logger.error("No pacakge found for %s" % self.project_name)
             return 1
+        return 0
 
     def parse_search_spec(self, spec):
         """
@@ -712,7 +723,7 @@ class Yolk(object):
                      license=ZPL AND name=Cheetah
         @type spec: string
         
-        @returns:  
+        @returns:  tuple with spec and operator
         """
 
         usage = \
@@ -766,7 +777,7 @@ class Yolk(object):
         return (spec, operator)
 
 
-    def pypi_search(self, spec):
+    def pypi_search(self):
         """
         Search PyPI by metadata keyword
         e.g. yolk -S name=yolk AND license=GPL
@@ -779,17 +790,17 @@ class Yolk(object):
           ["license=GPL"]
           ["name=yolk", "AND", "license=GPL"]
 
-        @returns: None
-
+        @returns: 0 on success or 1 if mal-formed search spec
 
         """
-        #Add remainging cli arguments to options.search
-        search_arg = self.options.search
+        spec = self.pkg_spec
+        #Add remainging cli arguments to options.pypi_search
+        search_arg = self.options.pypi_search
         spec.insert(0, search_arg.strip())
 
         (spec, operator) = self.parse_search_spec(spec)
         if not spec:
-            return 2
+            return 1
         for pkg in self.pypi.search(spec, operator):
             if pkg['summary']:
                 summary = pkg['summary'].encode('utf-8')
@@ -799,7 +810,7 @@ class Yolk(object):
         %s
     """ % (pkg['name'].encode('utf-8'), pkg["version"],
                     summary)
-
+        return 0
 
     def show_entry_map(self):
         """
@@ -808,27 +819,27 @@ class Yolk(object):
         @param dist: package
         @param type: srting
         
-        @returns: None or 2 if error
+        @returns: 0 for success or 1 if error
         """
         pprinter = pprint.PrettyPrinter()
         try:
             pprinter.pprint(pkg_resources.get_entry_map(self.options.entry_map))
-            return 0
         except pkg_resources.DistributionNotFound:
             self.logger.error("Distribution not found: %s" \
                     % self.options.entry_map)
-            return 2
+            return 1
+        return 0
 
     def show_entry_points(self):
         """
         Show entry points for a module
         
-        @returns: None or 2 if error
+        @returns: 0 for success or 1 if error
         
         """
         found = False
         for entry_point in \
-                pkg_resources.iter_entry_points(self.options.entry_points):
+                pkg_resources.iter_entry_points(self.options.show_entry_points):
             found = True
             try:
                 plugin = entry_point.load()
@@ -841,26 +852,33 @@ class Yolk(object):
                 pass
         if not found:
             self.logger.error("No entry points found for %s" \
-                    % self.options.entry_points)
-            return 2
+                    % self.options.show_entry_points)
+            return 1
+        return 0
 
-    def parse_pkg_ver(self, package_spec, want_installed):
+    def yolk_version(self):
+        """
+        Show yolk's version
+
+        @returns: 0
+        """
+        self.logger.info("yolk version %s (rev. %s)" % (VERSION, __revision__))
+        return 0
+
+    def parse_pkg_ver(self, want_installed):
         """
         Return tuple with project_name and version from CLI args
         If the user gave the wrong case for the project name, this corrects it
 
-        @param package_spec:
-        @type package_spec:
-        
         @param want_installed: whether package we want is installed or not
         @type want_installed: boolean
 
-        @returns: tuple(project_name, version) 
+        @returns: tuple(project_name, version, all_versions) 
         
         """
         all_versions = []
 
-        arg_str = ("").join(package_spec)
+        arg_str = ("").join(self.pkg_spec)
         if "==" not in arg_str:
             #No version specified
             project_name = arg_str
@@ -878,7 +896,9 @@ class Yolk(object):
                     self.pypi.query_versions_pypi(project_name)
 
             if not len(all_versions):
-                self.logger.error("I'm afraid we have no %s at The Cheese Shop. Perhaps a little Red Leicester?" % project_name)
+                msg = "I'm afraid we have no '%s' at " % project_name
+                msg += "The Cheese Shop. A little Red Leicester, perhaps?"
+                self.logger.error(msg)
                 sys.exit(2)
         return (project_name, version, all_versions)
 
@@ -896,7 +916,7 @@ def setup_opt_parser():
     opt_parser = optparse.OptionParser(usage=usage)
 
     opt_parser.add_option("--version", action='store_true', dest=
-                          "version", default=False, help=
+                          "yolk_version", default=False, help=
                           "Show yolk version and exit.")
 
     opt_parser.add_option("--debug", action='store_true', dest=
@@ -911,16 +931,16 @@ def setup_opt_parser():
             "The following options show information about installed Python packages. Activated packages are normal packages on sys.path that can be imported. Non-activated packages need 'pkg_resources.require()' before they can be imported, such as packages installed with 'easy_install --multi-version'. PKG_SPEC can be either a package name or package name and version e.g. Paste==0.9")
 
     group_local.add_option("-l", "--list", action='store_true', dest=
-                           "all", default=False, help=
+                           "show_all", default=False, help=
                            "List all Python packages installed by distutils or setuptools. Use PKG_SPEC to narrow results.")
 
     group_local.add_option("-a", "--activated", action='store_true',
-                           dest="active", default=False, help=
+                           dest="show_active", default=False, help=
                            'List activated packages installed by distutils or ' +
                            'setuptools. Use PKG_SPEC to narrow results.')
 
     group_local.add_option("-n", "--non-activated", action='store_true',
-                           dest="nonactive", default=False, help=
+                           dest="show_non_active", default=False, help=
                            'List non-activated packages installed by distutils or ' +
                            'setuptools. Use PKG_SPEC to narrow results.')
 
@@ -935,17 +955,17 @@ def setup_opt_parser():
                            '(use with -m or -M)')
 
     group_local.add_option("-d", "--depends", action='store', dest=
-                           "depends", metavar='PKG_SPEC',
+                           "show_deps", metavar='PKG_SPEC',
                            help= "Show dependencies for a package installed by " +
                            "setuptools if they are available.")
 
     group_local.add_option("--entry-points", action='store',
-                           dest="entry_points", default=False, help=
+                           dest="show_entry_points", default=False, help=
                            'List entry points for a module. e.g. --entry-points nose.plugins',
                             metavar="MODULE")
 
     group_local.add_option("--entry-map", action='store',
-                           dest="entry_map", default=False, help=
+                           dest="show_entry_map", default=False, help=
                            'List entry map for a package. e.g. --entry-map yolk',
                            metavar="PACKAGE_NAME")
     group_pypi = optparse.OptionGroup(opt_parser,
@@ -953,12 +973,12 @@ def setup_opt_parser():
             "The following options query the Python Package Index:")
 
     group_pypi.add_option("-C", "--changelog", action='store',
-                          dest="changelog", metavar='HOURS',
+                          dest="show_pypi_changelog", metavar='HOURS',
                           default=False, help=
                           "Show detailed ChangeLog for PyPI for last n hours. ")
 
     group_pypi.add_option("-D", "--download-links", action='store',
-                          metavar="PKG_SPEC", dest="download_links",
+                          metavar="PKG_SPEC", dest="show_download_links",
                           default=False, help=
                           "Show download URL's for package listed on PyPI. Use with -T to specify egg, source etc.")
 
@@ -972,8 +992,13 @@ def setup_opt_parser():
                           default=False, help=
                           "Launch web browser at home page for package.")
 
+    group_pypi.add_option("-I", "--pypi-index", action='store',
+                          dest="pypi_index",
+                          default=False, help=
+                          "Specify PyPI mirror for package index.")
+
     group_pypi.add_option("-L", "--latest-releases", action='store',
-                          dest="releases", metavar="HOURS",
+                          dest="show_pypi_releases", metavar="HOURS",
                           default=False, help=
                           "Show PyPI releases for last n hours. ")
 
@@ -982,7 +1007,7 @@ def setup_opt_parser():
                           metavar="PKG_SPEC", help=
                           "Show metadata for a package listed on PyPI. Use -f to show particular fields.")
 
-    group_pypi.add_option("-S", "", action="store", dest="search",
+    group_pypi.add_option("-S", "", action="store", dest="pypi_search",
                           default=False, help=
                           "Search PyPI by spec and optional AND/OR operator.",
                           metavar='SEARCH_SPEC <AND/OR SEARCH_SPEC>')
@@ -1035,10 +1060,10 @@ def validate_pypi_opts(opt_parser):
     (options, remaining_args) = opt_parser.parse_args()
     options_pkg_specs = [ options.versions_available, 
             options.query_metadata_pypi,
-            options.download_links,
+            options.show_download_links,
             options.browse_website,
             options.fetch,
-            options.depends,
+            options.show_deps,
             ]
     for pkg_spec in options_pkg_specs:
         if pkg_spec:
